@@ -10,6 +10,7 @@ class MyRawChecker(BaseChecker):
 
     name = 'custom_raw'
     msgs = {'W9901': ('use \\ for line continuation',
+                      'backslash-line-continuation',
                       ('Used when a \\ is used for a line continuation instead'
                        ' of using triple quoted string or parenthesis.')),
             }
@@ -18,11 +19,13 @@ class MyRawChecker(BaseChecker):
     def process_module(self, node):
         """process a module
 
-        the module's content is accessible via node.file_stream object
+        the module's content is accessible via node.stream() function
         """
-        for (lineno, line) in enumerate(node.file_stream):
-            if line.rstrip().endswith('\\'):
-                self.add_message('W9901', line=lineno)
+        with module.stream() as stream:
+            for (lineno, line) in enumerate(stream):
+                if line.rstrip().endswith('\\'):
+                    self.add_message('backslash-line-continuation',
+                                     line=lineno)
 
 
 def register(linter):
