@@ -19,6 +19,7 @@ import tokenize
 
 import astroid
 from astroid import bases
+
 from pylint import checkers, interfaces
 from pylint.utils import WarningScope
 from pylint.checkers import utils
@@ -50,7 +51,7 @@ def _check_dict_node(node):
 def _is_builtin(node):
     return getattr(node, 'name', None) in ('__builtin__', 'builtins')
 
-_accepts_iterator = {'iter', 'list', 'tuple', 'sorted', 'set', 'sum', 'any',
+_ACCEPTS_ITERATOR = {'iter', 'list', 'tuple', 'sorted', 'set', 'sum', 'any',
                      'all', 'enumerate', 'dict'}
 
 def _in_iterating_context(node):
@@ -71,12 +72,12 @@ def _in_iterating_context(node):
             return True
     # Various built-ins can take in an iterable or list and lead to the same
     # value.
-    elif isinstance(parent, astroid.CallFunc):
+    elif isinstance(parent, astroid.Call):
         if isinstance(parent.func, astroid.Name):
             parent_scope = parent.func.lookup(parent.func.name)[0]
-            if _is_builtin(parent_scope) and parent.func.name in _accepts_iterator:
+            if _is_builtin(parent_scope) and parent.func.name in _ACCEPTS_ITERATOR:
                 return True
-        elif isinstance(parent.func, astroid.Getattr):
+        elif isinstance(parent.func, astroid.Attribute):
             if parent.func.attrname == 'join':
                 return True
     # If the call is in an unpacking, there's no need to warn,
@@ -130,6 +131,11 @@ class Python3Checker(checkers.BaseChecker):
                   {'scope': WarningScope.NODE,
                    'maxversion': (3, 0),
                    'old_names': [('W0333', 'backtick')]}),
+        'E1609': ('Import * only allowed at module level',
+                  'import-star-module-level',
+                  'Used when the import star syntax is used somewhere '
+                  'else than the module level.',
+                  {'maxversion': (3, 0)}),
         'W1601': ('apply built-in referenced',
                   'apply-builtin',
                   'Used when the apply built-in function is referenced '
@@ -242,7 +248,7 @@ class Python3Checker(checkers.BaseChecker):
                   "Used when an object's next() method is called "
                   '(Python 3 uses the next() built-in function)',
                   {'maxversion': (3, 0)}),
-        'W1623': ("Assigning to a class' __metaclass__ attribute",
+        'W1623': ("Assigning to a class's __metaclass__ attribute",
                   'metaclass-assignment',
                   "Used when a metaclass is specified by assigning to __metaclass__ "
                   '(Python 3 specifies the metaclass as a class statement argument)',
@@ -378,7 +384,7 @@ class Python3Checker(checkers.BaseChecker):
         self._future_division = False
         self._future_absolute_import = False
 
-    def visit_function(self, node):
+    def visit_functiondef(self, node):
         if node.is_method() and node.name in self._unused_magic_methods:
             method_name = node.name
             if node.name.startswith('__'):
@@ -403,8 +409,8 @@ class Python3Checker(checkers.BaseChecker):
     def visit_print(self, node):
         self.add_message('print-statement', node=node)
 
-    @utils.check_messages('no-absolute-import')
-    def visit_from(self, node):
+    @utils.check_messages('no-absolute-import', 'import-star-module-level')
+    def visit_importfrom(self, node):
         if node.modname == '__future__':
             for name, _ in node.names:
                 if name == 'division':
@@ -413,6 +419,9 @@ class Python3Checker(checkers.BaseChecker):
                     self._future_absolute_import = True
         elif not self._future_absolute_import:
             self.add_message('no-absolute-import', node=node)
+        if node.names[0][0] == '*':
+            if not isinstance(node.scope(), astroid.Module):
+                self.add_message('import-star-module-level', node=node)
 
     @utils.check_messages('no-absolute-import')
     def visit_import(self, node):
@@ -420,7 +429,7 @@ class Python3Checker(checkers.BaseChecker):
             self.add_message('no-absolute-import', node=node)
 
     @utils.check_messages('metaclass-assignment')
-    def visit_class(self, node):
+    def visit_classdef(self, node):
         if '__metaclass__' in node.locals:
             self.add_message('metaclass-assignment', node=node)
 
@@ -435,8 +444,8 @@ class Python3Checker(checkers.BaseChecker):
 
     def _check_cmp_argument(self, node):
         # Check that the `cmp` argument is used
-        args = []
-        if (isinstance(node.func, astroid.Getattr)
+        kwargs = []
+        if (isinstance(node.func, astroid.Attribute)
                 and node.func.attrname == 'sort'):
             inferred = utils.safe_infer(node.func.expr)
             if not inferred:
@@ -445,28 +454,28 @@ class Python3Checker(checkers.BaseChecker):
             builtins_list = "{}.list".format(bases.BUILTINS)
             if (isinstance(inferred, astroid.List)
                     or inferred.qname() == builtins_list):
-                args = node.args
+                kwargs = node.keywords
 
         elif (isinstance(node.func, astroid.Name)
-                and node.func.name == 'sorted'):
+              and node.func.name == 'sorted'):
             inferred = utils.safe_infer(node.func)
             if not inferred:
                 return
 
             builtins_sorted = "{}.sorted".format(bases.BUILTINS)
             if inferred.qname() == builtins_sorted:
-                args = node.args
+                kwargs = node.keywords
 
-        for arg in args:
-            if isinstance(arg, astroid.Keyword) and arg.arg == 'cmp':
+        for kwarg in kwargs or []:
+            if kwarg.arg == 'cmp':
                 self.add_message('using-cmp-argument', node=node)
                 return
 
-    def visit_callfunc(self, node):
+    def visit_call(self, node):
         self._check_cmp_argument(node)
 
-        if isinstance(node.func, astroid.Getattr):
-            if any([node.args, node.starargs, node.kwargs]):
+        if isinstance(node.func, astroid.Attribute):
+            if any([node.args, node.keywords]):
                 return
             if node.func.attrname == 'next':
                 self.add_message('next-method-called', node=node)
@@ -504,7 +513,7 @@ class Python3Checker(checkers.BaseChecker):
             self.add_message('unpacking-in-except', node=node)
 
     @utils.check_messages('backtick')
-    def visit_backquote(self, node):
+    def visit_repr(self, node):
         self.add_message('backtick', node=node)
 
     @utils.check_messages('raising-string', 'old-raise-syntax')
