@@ -23,15 +23,14 @@ import tokenize
 import string
 import numbers
 
-import astroid
+import six
 
+import astroid
 from pylint.interfaces import ITokenChecker, IAstroidChecker, IRawChecker
 from pylint.checkers import BaseChecker, BaseTokenChecker
 from pylint.checkers import utils
 from pylint.checkers.utils import check_messages
 
-import six
-
 
 _PY3K = sys.version_info[:2] >= (3, 0)
 _PY27 = sys.version_info[:2] == (2, 7)
@@ -114,12 +113,12 @@ MSGS = {
               {'minversion': (2, 7)})
     }
 
-OTHER_NODES = (astroid.Const, astroid.List, astroid.Backquote,
-               astroid.Lambda, astroid.Function,
-               astroid.ListComp, astroid.SetComp, astroid.GenExpr)
+OTHER_NODES = (astroid.Const, astroid.List, astroid.Repr,
+               astroid.Lambda, astroid.FunctionDef,
+               astroid.ListComp, astroid.SetComp, astroid.GeneratorExp)
 
 if _PY3K:
-    import _string
+    import _string # pylint: disable=wrong-import-position, wrong-import-order
 
     def split_format_field_names(format_string):
         return _string.formatter_field_name_split(format_string)
@@ -157,9 +156,18 @@ def collect_string_fields(format_string):
             if nested:
                 for field in collect_string_fields(nested):
                     yield field
-    except ValueError:
-        # probably the format string is invalid
-        # should we check the argument of the ValueError?
+    except ValueError as exc:
+        # Probably the format string is invalid.
+        if exc.args[0].startswith("cannot switch from manual"):
+            # On Jython, parsing a string with both manual
+            # and automatic positions will fail with a ValueError,
+            # while on CPython it will simply return the fields,
+            # the validation being done in the interpreter (?).
+            # We're just returning two mixed fields in order
+            # to trigger the format-combined-specification check.
+            yield ""
+            yield "1"
+            return
         raise utils.IncompleteFormatString(format_string)
 
 def parse_format_method_string(format_string):
@@ -189,19 +197,18 @@ def parse_format_method_string(format_string):
     return keys, num_args, len(manual_pos_arg)
 
 def get_args(callfunc):
-    """ Get the arguments from the given `CallFunc` node.
+    """Get the arguments from the given `CallFunc` node.
+
     Return a tuple, where the first element is the
     number of positional arguments and the second element
     is the keyword arguments in a dict.
     """
-    positional = 0
-    named = {}
-
-    for arg in callfunc.args:
-        if isinstance(arg, astroid.Keyword):
-            named[arg.arg] = utils.safe_infer(arg.value)
-        else:
-            positional += 1
+    if callfunc.keywords:
+        named = {arg.arg: utils.safe_infer(arg.value)
+                 for arg in callfunc.keywords}
+    else:
+        named = {}
+    positional = len(callfunc.args)
     return positional, named
 
 def get_access_path(key, parts):
@@ -323,7 +330,7 @@ class StringMethodsChecker(BaseChecker):
         }
 
     @check_messages(*(MSGS.keys()))
-    def visit_callfunc(self, node):
+    def visit_call(self, node):
         func = utils.safe_infer(node.func)
         if (isinstance(func, astroid.BoundMethod)
                 and isinstance(func.bound, astroid.Instance)
@@ -352,7 +359,7 @@ class StringMethodsChecker(BaseChecker):
         #
         #    fmt = 'some string {}'.format
         #    fmt('arg')
-        if (isinstance(node.func, astroid.Getattr)
+        if (isinstance(node.func, astroid.Attribute)
                 and not isinstance(node.func.expr, astroid.Const)):
             return
         try:
@@ -361,8 +368,10 @@ class StringMethodsChecker(BaseChecker):
             return
         if not isinstance(strnode, astroid.Const):
             return
+        if not isinstance(strnode.value, six.string_types):
+            return
+
         if node.starargs or node.kwargs:
-            # TODO: Don't complicate the logic, skip these for now.
             return
         try:
             positional, named = get_args(node)
@@ -482,6 +491,8 @@ class StringMethodsChecker(BaseChecker):
                             previous = previous.getitem(specifier)
                         except (IndexError, TypeError):
                             warn_error = True
+                        except astroid.InferenceError:
+                            break
                     else:
                         try:
                             # Lookup __getitem__ in the current node,
