@@ -5,12 +5,12 @@ abstract methods.
 
 # pylint: disable=too-few-public-methods, missing-docstring
 # pylint: disable=no-absolute-import, metaclass-assignment
-# pylint: disable=abstract-method
-
-__revision__ = 0
+# pylint: disable=abstract-method, import-error, wildcard-import
 
 import abc
 from abc import ABCMeta
+from lala import Bala
+
 
 class GoodClass(object):
     __metaclass__ = abc.ABCMeta
@@ -56,6 +56,20 @@ class FourthBadClass(ThirdBadClass):
     pass
 
 
+class SomeMetaclass(object):
+    __metaclass__ = ABCMeta
+
+    @abc.abstractmethod
+    def prop(self):
+        pass
+
+class FifthGoodClass(SomeMetaclass):
+    """Don't consider this abstract if some attributes are
+    there, but can't be inferred.
+    """
+    prop = Bala # missing
+
+
 def main():
     """ do nothing """
     GoodClass()
@@ -66,3 +80,4 @@ def main():
     SecondBadClass() # [abstract-class-instantiated]
     ThirdBadClass() # [abstract-class-instantiated]
     FourthBadClass() # [abstract-class-instantiated]
+
