@@ -1,16 +1,16 @@
 # pylint:disable=C0103, print-statement, no-absolute-import
 """ho ho ho"""
-__revision__ = 'toto'
-
+from __future__ import print_function
 import sys
+__revision__ = 'toto'
 
 e = 1
 e2 = 'yo'
 e3 = None
 try:
     raise e
-except Exception, ex:
-    print ex
+except Exception as ex:
+    print(ex)
     _, _, tb = sys.exc_info()
 
 
@@ -23,8 +23,8 @@ def reraise():
     """reraise a catched exception instance"""
     try:
         raise Exception()
-    except Exception, exc:
-        print exc
+    except Exception as exc:
+        print(exc)
         raise exc
 
 raise e3
