@@ -22,8 +22,7 @@ from os.path import join, basename, dirname, isdir, abspath, sep
 import unittest
 
 import six
-
-from logilab.common.compat import reload
+from six.moves import reload_module
 
 from pylint import config, lint
 from pylint.lint import PyLinter, Run, preprocess_options, \
@@ -34,12 +33,34 @@ from pylint.utils import MSG_STATE_SCOPE_CONFIG, MSG_STATE_SCOPE_MODULE, MSG_STA
 from pylint.testutils import TestReporter
 from pylint.reporters import text, html
 from pylint import checkers
+from pylint.checkers.utils import check_messages
 from pylint import interfaces
 
-if sys.platform == 'win32':
-    HOME = 'USERPROFILE'
+if os.name == 'java':
+    if os._name == 'nt':
+        HOME = 'USERPROFILE'
+    else:
+        HOME = 'HOME'
 else:
-    HOME = 'HOME'
+    if sys.platform == 'win32':
+        HOME = 'USERPROFILE'
+    else:
+        HOME = 'HOME'
+
+@contextmanager
+def fake_home():
+    folder = tempfile.mkdtemp('fake-home')
+    old_home = os.environ.get(HOME)
+    try:
+        os.environ[HOME] = folder
+        yield
+    finally:
+        os.environ.pop('PYLINTRC', '')
+        if old_home is None:
+            del os.environ[HOME]
+        else:
+            os.environ[HOME] = old_home
+        rmtree(folder, ignore_errors=True)
 
 def remove(file):
     try:
@@ -47,15 +68,6 @@ def remove(file):
     except OSError:
         pass
 
-class GetNoteMessageTC(unittest.TestCase):
-    def test(self):
-        msg = None
-        for note in range(-1, 11):
-            note_msg = config.get_note_message(note)
-            self.assertNotEqual(msg, note_msg)
-            msg = note_msg
-
-
 HERE = abspath(dirname(__file__))
 INPUTDIR = join(HERE, 'input')
 
@@ -206,6 +218,22 @@ class PyLinterTC(unittest.TestCase):
         linter.file_state = FileState('toto')
         return linter
 
+    def test_pylint_visit_method_taken_in_account(self):
+        class CustomChecker(checkers.BaseChecker):
+            __implements__ = interfaces.IAstroidChecker
+            name = 'custom'
+            msgs = {'W9999': ('', 'custom', '')}
+
+            @check_messages('custom')
+            def visit_class(self, _):
+               pass
+
+        self.linter.register_checker(CustomChecker(self.linter))
+        self.linter.open()
+        out = six.moves.StringIO()
+        self.linter.set_reporter(text.TextReporter(out))
+        self.linter.check('abc')
+
     def test_enable_message(self):
         linter = self.init_linter()
         self.assertTrue(linter.is_message_enabled('W0101'))
@@ -225,32 +253,32 @@ class PyLinterTC(unittest.TestCase):
     def test_enable_message_category(self):
         linter = self.init_linter()
         self.assertTrue(linter.is_message_enabled('W0101'))
-        self.assertTrue(linter.is_message_enabled('C0121'))
+        self.assertTrue(linter.is_message_enabled('C0202'))
         linter.disable('W', scope='package')
         linter.disable('C', scope='module', line=1)
         self.assertFalse(linter.is_message_enabled('W0101'))
-        self.assertTrue(linter.is_message_enabled('C0121'))
-        self.assertFalse(linter.is_message_enabled('C0121', line=1))
+        self.assertTrue(linter.is_message_enabled('C0202'))
+        self.assertFalse(linter.is_message_enabled('C0202', line=1))
         linter.set_current_module('tutu')
         self.assertFalse(linter.is_message_enabled('W0101'))
-        self.assertTrue(linter.is_message_enabled('C0121'))
+        self.assertTrue(linter.is_message_enabled('C0202'))
         linter.enable('W', scope='package')
         linter.enable('C', scope='module', line=1)
         self.assertTrue(linter.is_message_enabled('W0101'))
-        self.assertTrue(linter.is_message_enabled('C0121'))
-        self.assertTrue(linter.is_message_enabled('C0121', line=1))
+        self.assertTrue(linter.is_message_enabled('C0202'))
+        self.assertTrue(linter.is_message_enabled('C0202', line=1))
 
     def test_message_state_scope(self):
         class FakeConfig(object):
             confidence = ['HIGH']
 
         linter = self.init_linter()
-        linter.disable('C0121')
+        linter.disable('C0202')
         self.assertEqual(MSG_STATE_SCOPE_CONFIG,
-                         linter.get_message_state_scope('C0121'))
+                         linter.get_message_state_scope('C0202'))
         linter.disable('W0101', scope='module', line=3)
         self.assertEqual(MSG_STATE_SCOPE_CONFIG,
-                         linter.get_message_state_scope('C0121'))
+                         linter.get_message_state_scope('C0202'))
         self.assertEqual(MSG_STATE_SCOPE_MODULE,
                          linter.get_message_state_scope('W0101', 3))
         linter.enable('W0102', scope='module', line=3)
@@ -364,16 +392,6 @@ class PyLinterTC(unittest.TestCase):
         for f in files:
             self.assertTrue(os.path.exists(f))
 
-    def test_lint_should_analyze_file(self):
-        self.linter.set_reporter(text.TextReporter())
-        self.linter.config.files_output = True
-        self.linter.should_analyze_file = lambda *args: False
-        self.addCleanup(remove, 'pylint_logilab.txt')
-
-        self.linter.check('logilab')
-        self.assertTrue(os.path.exists('pylint_logilab.txt'))
-        self.assertFalse(os.path.exists('pylint_logilab_common.txt'))
-
     def test_enable_report(self):
         self.assertEqual(self.linter.report_is_enabled('RP0001'), True)
         self.linter.disable('RP0001')
@@ -424,7 +442,7 @@ class PyLinterTC(unittest.TestCase):
         self.linter.error_mode()
         checkers = self.linter.prepare_checkers()
         checker_names = set(c.name for c in checkers)
-        should_not = set(('design', 'format', 'imports', 'metrics',
+        should_not = set(('design', 'format', 'metrics',
                       'miscellaneous', 'similarities'))
         self.assertSetEqual(set(), should_not & checker_names)
 
@@ -438,8 +456,7 @@ class PyLinterTC(unittest.TestCase):
         self.linter.set_option('reports', False)
         self.linter.set_option('disable', 'R,C,W')
         checker_names = [c.name for c in self.linter.prepare_checkers()]
-        for cname in  ('design', 'metrics', 'similarities',
-                       'imports'): # as a Fatal message that should be ignored
+        for cname in  ('design', 'metrics', 'similarities'):
             self.assertFalse(cname in checker_names, cname)
 
     def test_addmessage(self):
@@ -502,7 +519,7 @@ class ConfigTC(unittest.TestCase):
             pylintd = join(tempfile.gettempdir(), '.pylint.d')
             os.environ['PYLINTHOME'] = pylintd
             try:
-                reload(config)
+                reload_module(config)
                 self.assertEqual(config.PYLINT_HOME, pylintd)
             finally:
                 try:
@@ -513,38 +530,30 @@ class ConfigTC(unittest.TestCase):
             del os.environ['PYLINTHOME']
 
     def test_pylintrc(self):
-        fake_home = tempfile.mkdtemp('fake-home')
-        home = os.environ[HOME]
-        try:
-            os.environ[HOME] = fake_home
-            self.assertEqual(config.find_pylintrc(), None)
-            os.environ['PYLINTRC'] = join(tempfile.gettempdir(), '.pylintrc')
-            self.assertEqual(config.find_pylintrc(), None)
-            os.environ['PYLINTRC'] = '.'
-            self.assertEqual(config.find_pylintrc(), None)
-        finally:
-            os.environ.pop('PYLINTRC', '')
-            os.environ[HOME] = home
-            rmtree(fake_home, ignore_errors=True)
-            reload(config)
+        with fake_home():
+            try:
+                self.assertEqual(config.find_pylintrc(), None)
+                os.environ['PYLINTRC'] = join(tempfile.gettempdir(),
+                                              '.pylintrc')
+                self.assertEqual(config.find_pylintrc(), None)
+                os.environ['PYLINTRC'] = '.'
+                self.assertEqual(config.find_pylintrc(), None)
+            finally:
+                reload_module(config)
 
     def test_pylintrc_parentdir(self):
         with tempdir() as chroot:
 
             create_files(['a/pylintrc', 'a/b/__init__.py', 'a/b/pylintrc',
-                          'a/b/c/__init__.py', 'a/b/c/d/__init__.py'])
-            fake_home = tempfile.mkdtemp('fake-home')
-            home = os.environ[HOME]
-            try:
-                os.environ[HOME] = fake_home
+                          'a/b/c/__init__.py', 'a/b/c/d/__init__.py',
+                          'a/b/c/d/e/.pylintrc'])
+            with fake_home():
                 self.assertEqual(config.find_pylintrc(), None)
-            finally:
-                os.environ[HOME] = home
-                os.rmdir(fake_home)
             results = {'a'       : join(chroot, 'a', 'pylintrc'),
                        'a/b'     : join(chroot, 'a', 'b', 'pylintrc'),
                        'a/b/c'   : join(chroot, 'a', 'b', 'pylintrc'),
                        'a/b/c/d' : join(chroot, 'a', 'b', 'pylintrc'),
+                       'a/b/c/d/e' : join(chroot, 'a', 'b', 'c', 'd', 'e', '.pylintrc'),
                        }
             for basedir, expected in results.items():
                 os.chdir(join(chroot, basedir))
@@ -552,10 +561,7 @@ class ConfigTC(unittest.TestCase):
 
     def test_pylintrc_parentdir_no_package(self):
         with tempdir() as chroot:
-            fake_home = tempfile.mkdtemp('fake-home')
-            home = os.environ[HOME]
-            os.environ[HOME] = fake_home
-            try:
+            with fake_home():
                 create_files(['a/pylintrc', 'a/b/pylintrc', 'a/b/c/d/__init__.py'])
                 self.assertEqual(config.find_pylintrc(), None)
                 results = {'a'       : join(chroot, 'a', 'pylintrc'),
@@ -566,9 +572,6 @@ class ConfigTC(unittest.TestCase):
                 for basedir, expected in results.items():
                     os.chdir(join(chroot, basedir))
                     self.assertEqual(config.find_pylintrc(), expected)
-            finally:
-                os.environ[HOME] = home
-                rmtree(fake_home, ignore_errors=True)
 
 
 class PreprocessOptionsTC(unittest.TestCase):
@@ -626,14 +629,7 @@ class MessagesStoreTC(unittest.TestCase):
         self.store.register_messages(Checker())
 
     def _compare_messages(self, desc, msg, checkerref=False):
-        # replace \r\n with \n, because
-        # logilab.common.textutils.normalize_text
-        # uses os.linesep, which will
-        # not properly compare with triple
-        # quoted multilines used in these tests
-        self.assertMultiLineEqual(
-            desc,
-            msg.format_help(checkerref=checkerref).replace('\r\n', '\n'))
+        self.assertMultiLineEqual(desc, msg.format_help(checkerref=checkerref))
 
     def test_check_message_id(self):
         self.assertIsInstance(self.store.check_message_id('W1234'),
@@ -690,6 +686,6 @@ class MessagesStoreTC(unittest.TestCase):
         self.assertEqual('msg-symbol',
                          self.store.check_message_id('old-symbol').symbol)
 
-
+   
 if __name__ == '__main__':
     unittest.main()
